@@ -13,11 +13,13 @@ import logging
 import socket
 from collections import defaultdict, deque
 from queue import Queue
-from threading import Thread, Condition, RLock
+from threading import Condition, RLock, Thread
 from time import sleep
-from typing import Generic, List, Protocol, Tuple, TypeVar, runtime_checkable, cast
+from typing import Generic, List, Protocol, Tuple, TypeVar, cast, runtime_checkable
 
 from ..db.component_state import ComponentState
+from ..pdi.amc2_req import Amc2Req
+from ..pdi.base_req import BaseReq
 from ..protocol.command_def import CommandDefEnum
 from ..protocol.command_req import TMCC_FIRST_BYTE_TO_INTERPRETER, CommandReq
 from ..protocol.constants import (
@@ -25,14 +27,15 @@ from ..protocol.constants import (
     DEFAULT_BAUDRATE,
     DEFAULT_PORT,
     DEFAULT_QUEUE_SIZE,
-    DEFAULT_VALID_BAUDRATES,
-    CommandScope,
     DEFAULT_SERVER_PORT,
+    DEFAULT_VALID_BAUDRATES,
+    LOCO_TRACK_CRANE,
     PROGRAM_NAME,
+    CommandScope,
 )
 from ..protocol.multibyte.multibyte_constants import TMCC2_VARIABLE_INDEX
-from ..protocol.tmcc1.tmcc1_constants import TMCC1SyncCommandEnum, SyncCommandDef
-from ..protocol.tmcc2.tmcc2_constants import LEGACY_MULTIBYTE_COMMAND_PREFIX
+from ..protocol.tmcc1.tmcc1_constants import SyncCommandDef, TMCC1AuxCommandEnum, TMCC1SyncCommandEnum
+from ..protocol.tmcc2.tmcc2_constants import LEGACY_MULTIBYTE_COMMAND_PREFIX, TMCC2EngineCommandEnum
 from ..utils.ip_tools import get_ip_address
 
 log = logging.getLogger(__name__)
@@ -43,6 +46,21 @@ Topic = TypeVar("Topic")
 SYNCING = CommandReq(TMCC1SyncCommandEnum.SYNCHRONIZING)
 SYNC_COMPLETE = CommandReq(TMCC1SyncCommandEnum.SYNCHRONIZED)
 
+#
+# Certain Legacy/TMCC commands, such as the Engine Numerics, change core element state, such as
+# volume and rpm level. In these cases, we need to requery the Base 3 to get the updated
+# impacted state
+#
+COMMAND_IMPACTS = {
+    TMCC1AuxCommandEnum.RELATIVE_SPEED: (lambda x: x.is_amc2, Amc2Req.request_config),
+    TMCC1AuxCommandEnum.AUX1_OPT_ONE: (lambda x: x.is_amc2, Amc2Req.request_config),
+    TMCC1AuxCommandEnum.AUX2_OPT_ONE: (lambda x: x.is_amc2, Amc2Req.request_config),
+    TMCC2EngineCommandEnum.NUMERIC: (
+        lambda x: 1 <= x.address <= 99 and x.engine_type != LOCO_TRACK_CRANE,
+        BaseReq.request_config,
+    ),
+}
+
 
 class CommandListener(Thread):
     _instance: None = None
@@ -449,18 +467,24 @@ class CommandDispatcher(Thread, Generic[Topic, Message]):
                             log.debug(f"Filtering client update: {cmd}")
                         else:
                             self.publish_all(cmd)
-                    # if command is a legacy-style halt, just send to engines and trains
+                    # if command is a legacy-style halt, send to engines and trains
                     elif cmd.is_system_halt:
                         self.publish_all(cmd, [CommandScope.ENGINE, CommandScope.TRAIN])
                     # otherwise, send to the interested parties
                     else:
-                        if cmd.is_data is True:
+                        if cmd.is_data:
                             self.publish((cmd.scope, cmd.address, cmd.command, cmd.data), cmd)
                         self.publish((cmd.scope, cmd.address, cmd.command), cmd)
                         self.publish((cmd.scope, cmd.address), cmd)
                         self.publish(cmd.scope, cmd)
                     if self._broadcasts:
                         self.publish(BROADCAST_TOPIC, cmd)
+
+                    # We need to query state from the Base 3 in response to certain commands.
+                    # For example, if we receive a RELATIVE SPEED command on an AMC2, we have to
+                    # get the entire AMC2 config to update the new motor speed
+                    self.request_command_impact(cmd)
+
                     # update state on all clients
                     if self._server_port is not None:
                         """
@@ -534,7 +558,7 @@ class CommandDispatcher(Thread, Generic[Topic, Message]):
             if client_ip == client:
                 node_scope = cast(SyncCommandDef, option.command_def).is_node_scope
                 self.update_client_state(option, client=client, port=port)
-                if node_scope is True:
+                if node_scope:
                     return
 
     def signal_client(
@@ -612,7 +636,7 @@ class CommandDispatcher(Thread, Generic[Topic, Message]):
                             for state_packet in state_bytes:
                                 if not state_packet:
                                     continue
-                                if do_pause is True:
+                                if do_pause:
                                     sleep(0.01)
                                 try:
                                     self.send_state_packet(client_ip, client_port, state_packet)
@@ -621,7 +645,7 @@ class CommandDispatcher(Thread, Generic[Topic, Message]):
                                     log.warning(f"Exception sending state update {state} to {client_ip}:{client_port}")
                                     log.exception(e)
                         self._client_lock.notify_all()
-            # send sync complete message
+            # send sync_complete message
             self.send_state_packet(client_ip, client_port, EnqueueProxyRequests.sync_complete_response())
 
     def send_state_packet(self, client_ip: str, client_port: int, state: ComponentState | bytes):
@@ -658,7 +682,7 @@ class CommandDispatcher(Thread, Generic[Topic, Message]):
             with self._cv:
                 self._queue.put(cmd)
                 self._cv.notify_all()  # wake up receiving thread
-                if from_pdi is True:
+                if from_pdi:
                     pass  # TODO: prevent receiving same command from ser2 stream
 
     def shutdown(self) -> None:
@@ -693,7 +717,7 @@ class CommandDispatcher(Thread, Generic[Topic, Message]):
                 for channel in self._channels:
                     self._channels[channel].publish(message)
             else:
-                # send only to select channels and tuples with that channel
+                # only send to select channels and tuples with that channel
                 for channel in self._channels.keys():
                     if channel in channels or (isinstance(channel, tuple) and channel[0] in channels):
                         self._channels[channel].publish(message)
@@ -751,3 +775,15 @@ class CommandDispatcher(Thread, Generic[Topic, Message]):
             self._channels[BROADCAST_TOPIC].unsubscribe(subscriber)
             if not self._channels[BROADCAST_TOPIC].subscribers:
                 self._broadcasts = False
+
+    # noinspection PyArgumentList
+    @staticmethod
+    def request_command_impact(cmd: CommandReq) -> None:
+        action = COMMAND_IMPACTS.get(cmd.command, None)
+        if action:
+            from ..db.component_state_store import ComponentStateStore
+
+            state = ComponentStateStore.get_state(cmd.scope, cmd.address, create=False)
+            if state and action[0](state):
+                req = action[1](state, cmd)
+                req.send()
