@@ -13,11 +13,12 @@ import collections
 import logging
 import threading
 from collections import defaultdict
-from typing import List, Set, Tuple, TypeVar, Generic
+from typing import Generic, List, Set, Tuple, TypeVar, cast
 
 from ..comm.comm_buffer import CommBuffer
 from ..comm.command_listener import CommandListener, Message, Subscriber, Topic
 from ..db.client_state_listener import ClientStateListener
+from ..db.sync_state import SyncState
 from ..protocol.command_def import CommandDefEnum
 from ..protocol.command_req import CommandReq
 from ..protocol.constants import BROADCAST_ADDRESS, CommandScope
@@ -83,6 +84,15 @@ class ComponentStateStore:
             if cls._instance:
                 cls._instance._state.clear()
 
+    @classmethod
+    def is_state_synchronized(cls) -> bool:
+        if cls._instance is None:
+            return False
+        sync_state = cls._instance.get_state(CommandScope.SYNC, 99, False)
+        if sync_state is None:
+            return False
+        return cast(SyncState, sync_state).is_synchronized
+
     def __new__(cls, *args, **kwargs):
         """
         Provides singleton functionality. We only want one instance
@@ -341,9 +351,9 @@ class DependencyCache(Generic[T, C, E]):
         cmd_set = set()
         for cmd in commands:
             if isinstance(cmd, CommandDefEnum) and cmd.is_alias:
-                if include_aliases is True:
+                if include_aliases:
                     cmd_set.add(cmd)
-                if dereference_aliases is True:
+                if dereference_aliases:
                     cmd_set.add(cmd.alias)
             else:
                 cmd_set.add(cmd)
@@ -471,6 +481,7 @@ class DependencyCache(Generic[T, C, E]):
         # TMCC1 Engine Reset
         self.causes(
             Engine1.RESET,
+            Engine1.START_UP_IMMEDIATE,
             Engine1.SPEED_STOP_HOLD,
             Engine1.FORWARD_DIRECTION,
             Engine1.RPM_DOWN,
