@@ -26,7 +26,6 @@ from ..protocol.constants import (
     RPM_TYPE,
     SOUND_TYPE,
     STEAM_TYPE,
-    TRACK_CRANE_STATE_NUMERICS,
     CommandScope,
 )
 from ..protocol.multibyte.multibyte_constants import TMCC2EffectsControl
@@ -208,7 +207,7 @@ class EngineState(ComponentState):
 
     def decode_speed_info(self, speed_info):
         if speed_info is not None and speed_info == 255:  # not set
-            if self.is_legacy is True:
+            if self.is_legacy:
                 speed_info = 195
             else:
                 speed_info = 31
@@ -225,13 +224,18 @@ class EngineState(ComponentState):
             return
         with self._cv:
             # in the event the initial state hasn't been loaded from the base, resubmit the command
-            if self.is_comp_data_record is False:
+            if not self.is_comp_data_record:
                 if isinstance(command, CommandReq):
-                    from ..comm.command_listener import CommandDispatcher
+                    from src.pytrain import ComponentStateStore
 
-                    log.info(f"Still awaiting for initial state, will retry {command}...")
-                    CommandDispatcher.get().offer(command)
-                    return
+                    if not ComponentStateStore.is_state_synchronized:
+                        from ..comm.command_listener import CommandDispatcher
+
+                        log.info(f"Still awaiting for initial state, will retry {command}...")
+                        CommandDispatcher.get().offer(command)
+                        return
+                    else:
+                        self.initialize(self.scope, self.tmcc_id)
             super().update(command)
             if isinstance(command, CompDataMixin) and command.is_comp_data_record:
                 self._update_comp_data(command.comp_data)
@@ -244,7 +248,7 @@ class EngineState(ComponentState):
 
                 # handle some aspects of the halt command
                 if command.command == TMCC1HaltCommandEnum.HALT:
-                    if self.is_legacy is True:
+                    if self.is_legacy:
                         self._aux1 = TMCC2.AUX1_OFF
                         self._aux2 = TMCC2.AUX2_OFF
                         self._aux = TMCC2.AUX2_OPTION_ONE
@@ -252,9 +256,10 @@ class EngineState(ComponentState):
                         self._aux1 = TMCC1.AUX1_OFF
                         self._aux2 = TMCC1.AUX2_OFF
                         self._aux = TMCC1.AUX2_OPTION_ONE
-                    self.comp_data.speed = 0
-                    self.comp_data.rpm_tmcc = 0
-                    self.comp_data.labor_tmcc = 12
+                    if self.comp_data is not None:
+                        self.comp_data.speed = 0
+                        self.comp_data.rpm_tmcc = 0
+                        self.comp_data.labor_tmcc = 12
                     self._numeric = None
                     self._last_command = command
 
@@ -264,21 +269,8 @@ class EngineState(ComponentState):
 
                 # handle last numeric
                 if command.command in NUMERIC_SET:
-                    from ..pdi.base3_buffer import Base3Buffer
-
-                    if self.engine_type in {
-                        LOCO_TRACK_CRANE,
-                    }:
-                        if command.data in TRACK_CRANE_STATE_NUMERICS:
-                            self._numeric = command.data
-                            self._numeric_cmd = command.command
-                    else:
-                        self._numeric = command.data
-                        self._numeric_cmd = command.command
-                        # numeric commands can change RPM, Volume, and reset the train
-                        # force a state update for this engine/train, if we are connected
-                        # to a Base 3
-                        Base3Buffer.request_state_update(self.address, self.scope)
+                    self._numeric = command.data
+                    self._numeric_cmd = command.command
                 elif cmd_effects & NUMERIC_SET:
                     numeric = self._harvest_effect(cmd_effects & NUMERIC_SET)
                     log.info(f"What to do? {command}: {numeric} {type(numeric)}")
@@ -736,7 +728,7 @@ class EngineState(ComponentState):
     @property
     def is_legacy(self) -> bool:
         if self._is_legacy is None:
-            if self.scope == CommandScope.ENGINE and self.address > 99:
+            if self.scope in {CommandScope.ENGINE, CommandScope.TRAIN} and self.address > 99:
                 self._is_legacy = True
             elif self.control_type is not None:
                 if self.control_type == LEGACY_CONTROL_TYPE:
@@ -774,6 +766,7 @@ class TrainState(EngineState):
         if scope != CommandScope.TRAIN:
             raise ValueError(f"Invalid scope: {scope}, expected {CommandScope.TRAIN.name}")
         super().__init__(scope)
+        # TODO: FIXME!!
         # hard code TMCC2, for now
         self._is_legacy = True
 
