@@ -16,8 +16,15 @@ from enum import unique
 from typing import Dict, Tuple
 
 from ..command_def import CommandDef, CommandDefEnum
-from ..constants import CommandPrefix, CommandScope, CommandSyntax
-from ..constants import RELATIVE_SPEED_MAP, OfficialRRSpeeds, DEFAULT_ADDRESS
+from ..constants import (
+    DEFAULT_ADDRESS,
+    RELATIVE_SPEED_MAP,
+    REVERSE_SPEED_MAP,
+    CommandPrefix,
+    CommandScope,
+    CommandSyntax,
+    OfficialRRSpeeds,
+)
 
 
 class TMCC1Enum(CommandDefEnum):
@@ -285,12 +292,14 @@ TMCC1_ACC_REAR_COUPLER_COMMAND: int = 0x8006
 TMCC1_ACC_BOOST_COMMAND: int = 0x8004
 TMCC1_ACC_BRAKE_COMMAND: int = 0x8007
 TMCC1_ACC_RELATIVE_SPEED_COMMAND: int = 0x8040
+TMCC1_ACC_FORWARD_SPEED_COMMAND: int = 0x8000
+TMCC1_ACC_REVERSE_SPEED_COMMAND: int = 0x807C
 
 
 @unique
 class TMCC1AuxCommandEnum(TMCC1Enum):
     SET_ADDRESS = TMCC1CommandDef(TMCC1_ACC_SET_ADDRESS_COMMAND, TMCC1CommandIdentifier.ACC)
-    NUMERIC = TMCC1CommandDef(TMCC1_ACC_NUMERIC_COMMAND, TMCC1CommandIdentifier.ACC, d_max=9)
+    NUMERIC = TMCC1CommandDef(TMCC1_ACC_NUMERIC_COMMAND, TMCC1CommandIdentifier.ACC, d_max=9, aux1=True)
     AUX1_OFF = TMCC1CommandDef(TMCC1_ACC_AUX_1_OFF_COMMAND, TMCC1CommandIdentifier.ACC)
     AUX1_ON = TMCC1CommandDef(TMCC1_ACC_AUX_1_ON_COMMAND, TMCC1CommandIdentifier.ACC)
     AUX1_OPT_ONE = TMCC1CommandDef(TMCC1_ACC_AUX_1_OPTION_1_COMMAND, TMCC1CommandIdentifier.ACC)
@@ -303,6 +312,10 @@ class TMCC1AuxCommandEnum(TMCC1Enum):
     REAR_COUPLER = TMCC1CommandDef(TMCC1_ACC_REAR_COUPLER_COMMAND, TMCC1CommandIdentifier.ACC)
     BOOST = TMCC1CommandDef(TMCC1_ACC_BOOST_COMMAND, TMCC1CommandIdentifier.ACC)
     BRAKE = TMCC1CommandDef(TMCC1_ACC_BRAKE_COMMAND, TMCC1CommandIdentifier.ACC)
+    FORWARD_SPEED = TMCC1CommandDef(TMCC1_ACC_FORWARD_SPEED_COMMAND, TMCC1CommandIdentifier.ACC, d_max=3)
+    REVERSE_SPEED = TMCC1CommandDef(
+        TMCC1_ACC_REVERSE_SPEED_COMMAND, TMCC1CommandIdentifier.ACC, d_map=REVERSE_SPEED_MAP
+    )
     RELATIVE_SPEED = TMCC1CommandDef(
         TMCC1_ACC_RELATIVE_SPEED_COMMAND, TMCC1CommandIdentifier.ACC, d_map=RELATIVE_SPEED_MAP
     )
